package com.qgs.dc.mq.consumer;

import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSONObject;
import com.qgs.dc.common.utils.CommonFunction;
import com.qgs.dc.mq.Constant.Constant;
import com.qgs.dc.mq.configuration.ConfigOf00B;
import com.qgs.dc.mq.consumer.commonHandler.MQMessageHandler;
import com.qgs.dc.mq.entity.MQMessage;
import com.qgs.dc.mq.entity.common.Header;
import com.qgs.dc.mq.entity.specificBody.QueryEQStatusBody;
import com.qgs.dc.mq.secsgem.SendedList;
import com.rabbitmq.client.Channel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.rabbit.annotation.RabbitHandler;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.amqp.support.AmqpHeaders;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.messaging.Message;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * @Desc: "PID00B设备 接收MQ消息 监听类"
 * @Author: caixiang
 * @DATE: 2021/6/22 15:30
 *
 * Ctrl+R 替换设备名
 */
@Component
public class PID00BReceived {

    private static final Logger logger = LoggerFactory.getLogger(PID00BReceived.class);

    @Autowired
    MQMessageHandler mqMessageHandler;

    @RabbitListener(queues = ConfigOf00B.EAP_REQUEST_QUEUE_00B)
    @RabbitHandler
    public void eapRequest(Message<?> message, Channel channel)throws Exception{

        logger.info("==============received message-EAP_REQUEST_QUEUE_00B=================,priority:"+"equipmentName"+message.getHeaders().get("attr2"));

        Long deliveryTag = (Long)message.getHeaders().get(AmqpHeaders.DELIVERY_TAG);
        //MQMessage 中的 transactionId
        String transactionId = (String)message.getHeaders().get("spring_returned_message_correlation");
        //logger.info("transactionId:"+transactionId);

        try {

            MQMessage mqMessage = CommonFunction.parse(message);

            //1. 正常情况
            //Integer integer = mqMessageHandler.requestHandler(message);
            String integer= HttpUtil.post(Constant.mesCallBackUrl,JSONObject.toJSONString(mqMessage));
            Integer result = Integer.valueOf(integer);

            //注意 这里特别注意 已经拒收的消息 再签收是要出错的，这里要特别注意。
            if(result == 1){
                logger.info("在 EAP_REQUEST_QUEUE_00B 队列中,transitionId"+transactionId+", 这条消息处理成功");
                channel.basicAck(deliveryTag,false);
            }else {
                logger.error("在 EAP_REQUEST_QUEUE_00B 队列中,transitionId"+transactionId+" 处理消息的时候 出现异常，然后 拒签消息 ，然后丢到死信队列");
                channel.basicNack(deliveryTag,false,false);
            }


            //2.模拟异常 ，然后 拒签消息 ，然后丢到死信队列

            //throw new Exception("11111");
        }catch (Exception e){
            // 第一个false 是 不批量签收；第二个false 是 不重回队列
            channel.basicNack(deliveryTag,false,false);
            return;
        }



    }

    public static void main(String[] args) {
        //localhost:8001


        MQMessage mqMessage = new MQMessage();

        Header header = new Header("Request","Execute","QUERYEQPStatus","12");
        QueryEQStatusBody queryEQStatusBody = new QueryEQStatusBody();
        queryEQStatusBody.setVidType("u4");
        List<String> vids = new ArrayList<>();
        vids.add("10000");
        vids.add("10001");
        vids.add("10002");
        queryEQStatusBody.setVidList(vids);
        mqMessage.setBody(JSONObject.toJSONBytes(queryEQStatusBody));
        mqMessage.setHeader(header);


        String s = JSONObject.toJSONString(mqMessage);

        System.out.println(s);
        String result= HttpUtil.post("localhost:8001/receivedFromEapRequest",s);

        System.out.println(result);
    }

    @RabbitListener(queues = ConfigOf00B.MES_RESPONSE_QUEUE_00B)
    @RabbitHandler
    public void mesResponse(Message<?> message, Channel channel)throws Exception{
        logger.info("==============PID00B_Exchange-MES_Response_Queue=================,priority:"+message.getHeaders().get("priority")+",attr1"+message.getHeaders().get("attr1"));
        Long deliveryTag = (Long)message.getHeaders().get(AmqpHeaders.DELIVERY_TAG);

        //模拟异常 ，然后 拒签消息 ，然后丢到死信队列
        try {
            MQMessage mqMessage = CommonFunction.parse(message);
            String transactionId = mqMessage.getHeader().getTransactionId();
            Integer result = mqMessageHandler.responseHandler(message);
            if(result == 1){
                logger.info("在 MES_RESPONSE_QUEUE_00B 队列中 , transitionId"+transactionId+" 这条消息处理成功");
                channel.basicAck(deliveryTag,false);
            }else {
                logger.error("在 MES_RESPONSE_QUEUE_00B 队列中 ,transitionId"+transactionId+" 处理消息的时候 出现异常，然后 拒签消息 ，然后丢到死信队列");
                channel.basicNack(deliveryTag,false,false);
            }

            //throw new Exception("11111");
        }catch (Exception e){
            // 第一个false 是 不批量签收；第二个false 是 不重回队列
            channel.basicNack(deliveryTag,false,false);
            return;
        }

        //注意 这里特别注意 已经拒收的消息 再签收是要出错的，这里要特别注意。
        //channel.basicAck(deliveryTag,false);

    }

}
